local map -- stores tiledata
local mapWidth, mapHeight -- width and height in tiles

local mapX, mapY -- view x,y in tiles. can be a fractional value like 3.25.

local tilesDisplayWidth, tilesDisplayHeight -- number of tiles to show
local zoomX, zoomY

local tilesetImage
local tileSize -- size of tiles in pixels
local tileQuads = {} -- parts of the tileset used for different tiles
local tilesetSprite
local frame = 0
local animate_delay

function love.load()
  animate_delay = love.timer.getMicroTime();
  love.graphics.setMode( 288, 320, 0, 0, 0 )
  love.graphics.setBackgroundColor( 153, 255, 255 )

  setupMap()
  setupMapView()
  setupTileset()
  love.graphics.setFont(12)
  
end

function setupMap()
  mapWidth = 9
  mapHeight = 10
  
  map = {}
  map={
    { 8, 8, 8, 8, 8, 8, 8, 8, 1, 0 }, 
    { 8, 8, 8, 9, 8, 8, 8, 8, 1, 0 },
    { 8, 8, 8, 8, 8, 8, 8, 8, 8, 8 },
    { 8, 8, 8, 8, 8, 8, 8, 8, 1, 0 },
    { 8, 9, 8, 8, 8, 8, 8, 8, 1, 0 },
    { 8, 8, 8, 8, 8, 8, 8, 8, 1, 0 },
    { 8, 8, 8, 8, 8, 8, 8, 8, 1, 0 },
    { 8, 8, 8, 8, 8, 8, 8, 8, 1, 0 },
    { 8, 10, 8, 8, 8, 8, 8, 8, 1, 0 },
    { 8, 8, 8, 8, 8, 8, 8, 8, 1, 0 }
    }
end

function setupMapView()
  mapX = 1
  mapY = 1
  tilesDisplayWidth = 9
  tilesDisplayHeight = 10
  
  zoomX = 1
  zoomY = 1
end

function setupTileset()
  tilesetImage = love.graphics.newImage( "phil.png" )
  tilesetImage:setFilter("nearest", "linear") -- this "linear filter" removes some artifacts if we were to scale the tiles
  tileSize = 32
  tileSize = 32
  
  -- grass base
  tileQuads[0] = love.graphics.newQuad(4 * tileSize, 0, tileSize, tileSize,
  tilesetImage:getWidth(), tilesetImage:getHeight())
  -- grass top flat
  tileQuads[1] = love.graphics.newQuad(5 * tileSize, 0, tileSize, tileSize,
    tilesetImage:getWidth(), tilesetImage:getHeight())
  -- ending gate
  tileQuads[2] = love.graphics.newQuad(6 * tileSize, 0, tileSize, tileSize,
    tilesetImage:getWidth(), tilesetImage:getHeight())
  -- blank
  tileQuads[3] = love.graphics.newQuad(7 * tileSize, 0, tileSize, tileSize,
    tilesetImage:getWidth(), tilesetImage:getHeight())
  --Blue sky
  tileQuads[8] = love.graphics.newQuad(8 * tileSize, 0, tileSize, tileSize,
    tilesetImage:getWidth(), tilesetImage:getHeight())
 -- Popcorn or cloud 
 tileQuads[9] = love.graphics.newQuad(9 * tileSize, 0, tileSize, tileSize,
    tilesetImage:getWidth(), tilesetImage:getHeight())
 --Fuckin Crazy ass sun
 tileQuads[10] = love.graphics.newQuad(10 * tileSize, 0, tileSize, tileSize,
    tilesetImage:getWidth(), tilesetImage:getHeight())
 
 --The donkey frames
 tileQuads[4] = love.graphics.newQuad(0 * tileSize, 0 * tileSize, tileSize, tileSize,
 tilesetImage:getWidth(), tilesetImage:getHeight())
 tileQuads[5] = love.graphics.newQuad(1 * tileSize, 0 * tileSize, tileSize, tileSize,
 tilesetImage:getWidth(), tilesetImage:getHeight())
 tileQuads[6] = love.graphics.newQuad(2 * tileSize, 0 * tileSize, tileSize, tileSize,
 tilesetImage:getWidth(), tilesetImage:getHeight())
 tileQuads[7] = love.graphics.newQuad(3 * tileSize, 0 * tileSize, tileSize, tileSize,
 tilesetImage:getWidth(), tilesetImage:getHeight())

  tilesetBatch = love.graphics.newSpriteBatch(tilesetImage, tilesDisplayWidth * tilesDisplayHeight)
  CharacterTiles = love.graphics.newSpriteBatch(tilesetImage, tilesDisplayWidth * tilesDisplayHeight)
  updateTilesetBatch()
end

function updateTilesetBatch()
  tilesetBatch:clear()
  CharacterTiles:clear()
  for x=0, tilesDisplayWidth-1 do
     
     for y=0, tilesDisplayHeight-1 do
     
      tilesetBatch:addq(tileQuads[map[x+math.floor(mapX)][y+math.floor(mapY)]],
        x*tileSize, y*tileSize)
    end
  end
  if ((love.timer.getMicroTime())-animate_delay) > 0.3 then
     frame = frame + 1
     animate_delay = love.timer.getMicroTime()
  end
  if frame >= 4 then
     frame = 0
  end
  if frame == 0  then
     CharacterTiles:addq(tileQuads[4],0*tileSize,8*tileSize)
  elseif frame == 1  then
     CharacterTiles:addq(tileQuads[5],0*tileSize,8*tileSize)
  elseif frame == 2  then
     CharacterTiles:addq(tileQuads[6],0*tileSize,8*tileSize)
  elseif frame == 3  then
     CharacterTiles:addq(tileQuads[7],0*tileSize,8*tileSize)
  end
  
end

-- central function for moving the map
function moveMap(dx, dy)
  oldMapX = mapX
  oldMapY = mapY
  mapX = math.max(math.min(mapX + dx, mapWidth - tilesDisplayWidth), 1)
  mapY = math.max(math.min(mapY + dy, mapHeight - tilesDisplayHeight), 1)
  -- only update if we actually moved
  if math.floor(mapX) ~= math.floor(oldMapX) or math.floor(mapY) ~= math.floor(oldMapY) then
    updateTilesetBatch()
  end
end

function love.update(dt)
  if love.keyboard.isDown("up")  then
    moveMap(0, -0.2 * tileSize * dt)
  end
  if love.keyboard.isDown("down")  then
    moveMap(0, 0.2 * tileSize * dt)
  end
  if love.keyboard.isDown("left")  then
    moveMap(-0.2 * tileSize * dt, 0)
  end
  if love.keyboard.isDown("right")  then
    moveMap(0.2 * tileSize * dt, 0)
  end
end

function love.draw()
  
  love.graphics.draw(tilesetBatch,
    math.floor(-zoomX*(mapX%1)*tileSize), math.floor(-zoomY*(mapY%1)*tileSize),
    0, zoomX, zoomY)
  love.graphics.draw(CharacterTiles,
   math.floor(-zoomX*(mapX%1)*tileSize), math.floor(-zoomY*(mapY%1)*tileSize),
   0, zoomX, zoomY)
   updateTilesetBatch()
end

